close all;
clear all;

FontSize=16;
FontName='Times New Roman';

%% Undersampling med digitalt oscilloskop
%"RF"-signal fr undersampling, fs=625MHz
x1=textread('inn_2us_1_25GS.dat')';
%Fjerner eventuelle DC komponenter
x1=x1-mean(x1);
%Signal etter undersampling med fs=25MHz
tmp=textread('undersampel_100us_50MS.dat')';
x2=tmp(1:2500);%Vil kun ha 2500 punkter!
%Fjerner eventuelle DC komponenter
x2=x2-mean(x2);

%Definerer de tre forskjellige fs som er brukt
fs1=1.25e9;
fs2=50e6;
fs3=125e6;

%Finner nsket antall punkter som skal brukes i FFT-algoritmen
NFFT=2^(nextpow2(length(x1)));

%Beregner frekvensresponsen vha. FFT-algoritmen
X1=fft(x1.*window(@hann,length(x1))',NFFT);
X2=fft(x2.*window(@hann,length(x2))',NFFT);

%Finner riktig frekvensvektor til frekvensresponsen
f1=linspace(-0.5*fs1,0.5*fs1,NFFT);
f2=linspace(-0.5*fs2,0.5*fs2,NFFT);

%Plotter frekvensresponsen til RF-signalet fr undersampling
figure();
plot(f1./1e6,20*log10(fftshift(abs(X1)./max(abs(X1)))));
title('Frekvensspekter fr undersampling, fs=1.25GHz, NFFT=4096', 'FontSize', FontSize,'FontName',FontName);
xlabel('f [MHz]','FontSize', FontSize,'FontName',FontName);
ylabel('Normalisert spekter','FontSize', FontSize,'FontName',FontName);
axis([0 625 -80 3]);
set(gca, 'FontSize', FontSize,'FontName',FontName);

%Plotter frekvensresponsen etter undersampling med fs=50MHz
figure();
plot(f2./1e6,20*log10(fftshift(abs(X2)./max(abs(X2)))));
title('Frekvensspekter etter undersampling, fs=50MHz, NFFT=4096', 'FontSize', FontSize,'FontName',FontName);
xlabel('f [MHz]','FontSize', FontSize,'FontName',FontName);
ylabel('Normalisert spekter','FontSize', FontSize,'FontName',FontName);
axis([0 25 -80 3]);
set(gca, 'FontSize', FontSize,'FontName',FontName);


%Sjekker stynivet til oscilloskopet
%styvektor fra scoop
tmp=textread('terminert_100us_50MS.dat')';
x4=tmp(1:2500);%Vil kun ha 2500 punkter!
%Fjerner eventuelle DC komponenter
x4=x4-mean(x4);

%Beregner frekvensresponsen vha. FFT-algoritmen
X4=fft(x4.*window(@hann,length(x4))',NFFT);
%Plotter frekvensresponsen etter undersampling med fs=50MHz
figure();
plot(f2./1e6,20*log10(fftshift(abs(X2)./max(abs(X2)))));
hold on;
plot(f2./1e6,20*log10(fftshift(abs(X4)./max(abs(X2)))),'m');
title('Frekvensspekter etter undersampling, fs=50MHz, NFFT=4096', 'FontSize', FontSize,'FontName',FontName);
xlabel('f [MHz]','FontSize', FontSize,'FontName',FontName);
ylabel('Normalisert spekter','FontSize', FontSize,'FontName',FontName);
axis([0 25 -80 3]);
set(gca, 'FontSize', FontSize,'FontName',FontName);
hleg=legend('Undersamplet resultat','Styniv oscilloskop',2);
set(hleg,'FontSize', 12,'FontName','Times New Roman');
hold off;

%% Beregner SINAD i begge tilfellene

%Beregner frekvensresponsen med et Blackman-Harris windu
X1=fft(x1.*window(@blackmanharris,length(x1))');
X2=fft(x2.*window(@blackmanharris,length(x2))');

%Plotter frekvensresponsen for  se hvor signalkomponentene ligger
figure();
plot(20*log10(abs(X1)./abs(max(X1))));

%Definerer sty og signal effekter
sign_pow1=0;
noise_pow1=0;

sign_pow2=0;
noise_pow2=0;

%Gr igjennom "RF"-signalet og summerer signal og stykomponenter
for i=1:length(X1)
    if i>=413 && i<=421 || i>=2081 && i<=2089  
        sign_pow1=sign_pow1+(abs(X1(i))^2);
    else
        noise_pow1=noise_pow1+abs(X1(i))^2;
   end
end
%Gr igjennom det undersamplede resultatet og summerer signal og stykomponenter
for i=1:length(X1)
    if i>=397 && i<=405 || i>=2097 && i<=2105  
        sign_pow2=sign_pow2+(abs(X2(i))^2);
    else
        noise_pow2=noise_pow2+abs(X2(i))^2;
   end
end

%For  beregne SNR istedenfor SINAD
%noise_pow1=noise_pow1-2*(abs(X1(829))^2+abs(X1(830))^2+abs(X1(831))^2+abs(X1(832))^2+abs(X1(833))^2+abs(X1(834))^2+abs(X1(835))^2+abs(X1(836))^2+abs(X1(837))^2);
%noise_pow1=noise_pow1-2*(abs(X1(1245))^2+abs(X1(1246))^2+abs(X1(1247))^2+abs(X1(1248))^2+abs(X1(1249))^2+abs(X1(1250))^2+abs(X1(1251))^2);


%Beregner SINAD i begge tilfellene
SINAD1=10*log10(sign_pow1/noise_pow1);
SINAD2=10*log10(sign_pow2/noise_pow2);


%% Betydningen av jitter og analog bndbredde
 
%Punktprvingsfrekvens brukt denne testen
fs=250e6;

%Leser inn alle de undersamplede resultatene og fjerner DC
x50=textread('50MHz_250Ms_10us.dat')';
x50=x50-mean(x50);
x175=textread('175MHz_250Ms_10us.dat')';
x175=x175-mean(x175);
x300=textread('300MHz_250Ms_10us.dat')';
x300=x300-mean(x300);
x425=textread('425MHz_250Ms_10us.dat')';
x425=x425-mean(x425);
x550=textread('550MHz_250Ms_10us.dat')';
x550=x550-mean(x550);
x675=textread('675MHz_250Ms_10us.dat')';
x675=x675-mean(x675);
x800=textread('800MHz_250Ms_10us.dat')';
x800=x800-mean(x800);
x925=textread('925MHz_250Ms_10us.dat')';
x925=x925-mean(x925);
x1050=textread('1050MHz_250Ms_10us.dat')';
x1050=x1050-mean(x1050);

%Beregner frekvensresponsen med et Blackman-Harris windu
X50=fft(x50.*window(@blackmanharris,length(x50))');
X175=fft(x175.*window(@blackmanharris,length(x175))');
X300=fft(x300.*window(@blackmanharris,length(x300))');
X425=fft(x425.*window(@blackmanharris,length(x425))');
X550=fft(x550.*window(@blackmanharris,length(x550))');
X675=fft(x675.*window(@blackmanharris,length(x675))');
X800=fft(x800.*window(@blackmanharris,length(x800))');
X925=fft(x925.*window(@blackmanharris,length(x925))');
X1050=fft(x1050.*window(@blackmanharris,length(x1050))');

%Plotter frekvensresponsen for  se hvor signalkomponentene ligger
%figure();
%stem(abs(X425))

%Definerer sty og signal effekter
s50=0;n50=0;
s175=0;n175=0;
s300=0;n300=0;
s425=0;n425=0;
s550=0;n550=0;
s675=0;n675=0;
s800=0;n800=0;
s925=0;n925=0;
s1050=0;n1050=0;

%Gr igjennom alle frekvensresonsene og summerer signal og stykomponenter
for i=1:length(X50)

    if i>=497 && i<=505 || i>=1997 && i<=2005  
        s50=s50+abs(X50(i)^2);
        s300=s300+abs(X300(i)^2);
        s550=s550+abs(X550(i)^2);
        s800=s800+abs(X800(i)^2);
        s1050=s1050+abs(X1050(i)^2);
    else
        n50=n50+abs(X50(i)^2);
        n300=n300+abs(X300(i)^2);
        n550=n550+abs(X550(i)^2);
        n800=n800+abs(X800(i)^2);
        n1050=n1050+abs(X1050(i)^2);
    end
    if i>=747 && i<=755 || i>=1747 && i<=1755  
        s175=s175+abs(X175(i)^2);
        s425=s425+abs(X425(i)^2);
        s675=s675+abs(X675(i)^2);
        s925=s925+abs(X925(i)^2);
    else
        n175=n175+abs(X175(i)^2);
        n425=n425+abs(X425(i)^2);
        n675=n675+abs(X675(i)^2);
        n925=n925+abs(X925(i)^2);
    end
   
end

%Beregner SNR/SINAD i alle tilfellene
SNR50=10*log10(s50/n50);
SNR175=10*log10(s175/n175);
SNR300=10*log10(s300/n300);
SNR425=10*log10(s425/n425);
SNR550=10*log10(s550/n550);
SNR675=10*log10(s675/n675);
SNR800=10*log10(s800/n800);
SNR925=10*log10(s925/n925);
SNR1050=10*log10(s1050/n1050);

%Legger alle SNR resultatene i en matrise
SNR=[SNR50 SNR175 SNR300 SNR425 SNR550 SNR675 SNR800 SNR925 SNR1050];

%Plotter SNR resultatene sammen med teoretisk SNR med 8ps RMS jitter
figure();
f=[50 175 300 425 550 675 800 925 1050].*1e6;
stem(f./1e6,SNR);
hold on;
plot(f./1e6,-20*log10(2*pi*8e-12.*f),'r');
plot(f./1e6,-20*log10(2*pi*6e-12.*f),'c--');
plot(f./1e6,-20*log10(2*pi*5e-12.*f),'m--');
title('SINAD/SNR etter punktprving med fs=250MHz','FontSize', FontSize,'FontName',FontName);
xlabel('f [MHz]','FontSize', FontSize,'FontName',FontName);
ylabel('SINAD/SNR [dB]','FontSize', FontSize,'FontName',FontName);
axis([0 1100 20 40]);
set(gca, 'FontSize', FontSize,'FontName',FontName);
hleg=legend('SINAD','SNR \tau_{j}=8ps','SNR \tau_{j}=6ps','SNR \tau_{j}=5ps',4);
set(hleg,'FontSize', 12,'FontName','Times New Roman');
hold off;


%Definerer nsket lengde til FFT-algoritmen
NFFT=2^(nextpow2(length(x50)));

%Beregner frekvensresponsen for alle resultatene
X50=fft(x50.*window(@hann,length(x50))',NFFT);
X175=fft(x175.*window(@hann,length(x175))',NFFT);
X300=fft(x300.*window(@hann,length(x300))',NFFT);
X425=fft(x425.*window(@hann,length(x425))',NFFT);
X550=fft(x550.*window(@hann,length(x550))',NFFT);
X675=fft(x675.*window(@hann,length(x675))',NFFT);
X800=fft(x800.*window(@hann,length(x800))',NFFT);
X925=fft(x925.*window(@hann,length(x925))',NFFT);
X1050=fft(x1050.*window(@hann,length(x1050))',NFFT);

%Finner maksimalverdiene til alle frekvensresponsene
Maks_50=max(abs(X50));
Maks_175=max(abs(X175));
Maks_300=max(abs(X300));
Maks_425=max(abs(X425));
Maks_550=max(abs(X550));
Maks_675=max(abs(X675));
Maks_800=max(abs(X800));
Maks_925=max(abs(X925));
Maks_1050=max(abs(X1050));

%Normaliserer og tar logaritmen til alle maksimalverdiene
Demp_50=20*log10(Maks_50/Maks_50);
Demp_175=20*log10(Maks_175/Maks_50);
Demp_300=20*log10(Maks_300/Maks_50);
Demp_425=20*log10(Maks_425/Maks_50);
Demp_550=20*log10(Maks_550/Maks_50);
Demp_675=20*log10(Maks_675/Maks_50);
Demp_800=20*log10(Maks_800/Maks_50);
Demp_925=20*log10(Maks_925/Maks_50);
Demp_1050=20*log10(Maks_1050/Maks_50);

%Legger alle dempningene inn i samme matise
Demp=[Demp_50 Demp_175 Demp_300 Demp_425 Demp_550 Demp_675 Demp_800 Demp_925 Demp_1050];

figure();
stem(f./1e6,Demp);
title('Dempning som flge av den analoge bndbredden', 'FontSize', FontSize,'FontName',FontName);
xlabel('f [MHz]','FontSize', FontSize,'FontName',FontName);
ylabel('Dempning [dB]','FontSize', FontSize,'FontName',FontName);
axis([0 1100 -5 1]);
set(gca, 'FontSize', FontSize,'FontName',FontName);

%Definerer frekvensvektor til frekvensresponsen
f=linspace(-0.5*fs,0.5*fs,NFFT);

%Plotter frekvensresponsen til noen av resultatene og normaliserer til X50
%f_inn=50MHz
figure();
plot(f./1e6,20*log10(fftshift(abs(X50)./max(abs(X50)))));
title('Frekvensspekter, f_{inn}=50MHz, fs=250MHz, NFFT=4096', 'FontSize', FontSize,'FontName',FontName);
xlabel('f [MHz]','FontSize', FontSize,'FontName',FontName);
ylabel('Normalisert spekter','FontSize', FontSize,'FontName',FontName);
axis([0 125 -80 3]);
set(gca, 'FontSize', FontSize,'FontName',FontName);
%f_inn=675MHz
figure();
plot(f./1e6,20*log10(fftshift(abs(X675)./max(abs(X50)))));
title('Frekvensspekter, f_{inn}=675MHz, fs=250MHz, NFFT=4096', 'FontSize', FontSize,'FontName',FontName);
xlabel('f [MHz]','FontSize', FontSize,'FontName',FontName);
ylabel('Normalisert spekter','FontSize', FontSize,'FontName',FontName);
axis([0 125 -80 3]);
set(gca, 'FontSize', FontSize,'FontName',FontName);
%f_inn=1050MHz
figure();
plot(f./1e6,20*log10(fftshift(abs(X1050)./max(abs(X50)))));
title('Frekvensspekter, f_{inn}=1050MHz, fs=250MHz, NFFT=4096', 'FontSize', FontSize,'FontName',FontName);
xlabel('f [MHz]','FontSize', FontSize,'FontName',FontName);
ylabel('Normalisert spekter','FontSize', FontSize,'FontName',FontName);
axis([0 125 -80 3]);
set(gca, 'FontSize', FontSize,'FontName',FontName);
