close all;
clear all;

FontSize=16;
FontName='Times New Roman';

%% Sjekker frst bryterstyen nr BB er terminert med 50ohm
%Utgang uten innsignal med ufiltrert klokke
x1=textread('terminert_inngang_utenBPklokke_2.5GS_10us_115mVpkpk.dat')';
%Fjerner eventuelle DC komponenter
x1=x1-mean(x1);
%Utgang uten innsignal med filtrert klokke
x2=textread('terminert_inngang_medBPklokke_2.5GS_10us_115mVpkpk.dat')';
%Fjerner eventuelle DC komponenter
x2=x2-mean(x2);

fs=2.5e9;
t=linspace(0,10e-6,length(x1));

%plotter tidsresponsen for  vise over og undershooting (glitch)
figure();
plot(t(1:500)./1e-6,x1(1:500));
title('Tidsrespons, fs=2.5GHz', 'FontSize', FontSize,'FontName',FontName);
xlabel('t [\mus]','FontSize', FontSize,'FontName',FontName);
ylabel('x(t) [V]','FontSize', FontSize,'FontName',FontName);
set(gca, 'FontSize', FontSize,'FontName',FontName);

%Finner nsket antall punkter som skal brukes i FFT-algoritmen
NFFT=2^(nextpow2(length(x1)));

%Beregner frekvensresponsen vha. FFT-algoritmen
X1=fft(x1.*window(@hann,length(x1))',NFFT);
X2=fft(x2.*window(@hann,length(x2))',NFFT);

%Finner riktig frekvensvektor til frekvensresponsen
f=linspace(-0.5*fs,0.5*fs,NFFT);

%Plotter frekvensresponsen til utgangen uten filter p klokka
figure();
plot(f./1e6,20*log10(fftshift(abs(X1)./max(abs(X1)))));
title('Frekvensspekter uten filter, fs=2.5GHz, NFFT=32768', 'FontSize', FontSize,'FontName',FontName);
xlabel('f [MHz]','FontSize', FontSize,'FontName',FontName);
ylabel('|X(f)|','FontSize', FontSize,'FontName',FontName);
axis([0 500 -70 3]);
set(gca, 'FontSize', FontSize,'FontName',FontName);

%Plotter frekvensresponsen til utgangen med filter p klokka
figure();
plot(f./1e6,20*log10(fftshift(abs(X2)./max(abs(X2)))));
title('Frekvensspekter med filter, fs=2.5GHz, NFFT=32768', 'FontSize', FontSize,'FontName',FontName);
xlabel('f [MHz]','FontSize', FontSize,'FontName',FontName);
ylabel('|X(f)|','FontSize', FontSize,'FontName',FontName);
axis([0 500 -70 3]);
set(gca, 'FontSize', FontSize,'FontName',FontName);



%% Ser p utgangen til RTZ-kretsen med de to forskjellige koaksiallengdene

%Utgang RTZ-krets med koaksialkabel 1
x1=textread('RTZ_utgang_medium_coax_2.5GS_10us_600mVpkpk.dat')';
%Fjerner eventuelle DC komponenter
%x1=x1-mean(x1);
%Utgang RTZ-krets med koaksialkabel 2
x2=textread('RTZ_utgang_lang_coax_2.5GS_10us_600mVpkpk.dat')';
%Fjerner eventuelle DC komponenter
x2=x2-mean(x2);

fs=2.5e9;
t=linspace(0,10e-6,length(x1));

%nsket lengde til FFT-algoritmen
NFFT=2^(nextpow2(length(x1)));

%Beregner frekvensresponsen til RTZ-krets utgangene
X1=fft(x1.*window(@hann,length(x1))',NFFT);
X2=fft(x2.*window(@hann,length(x2))',NFFT);

%Frekvensvektor til frekvensreponsen
f=linspace(-0.5*(fs/1e6),0.5*(fs/1e6),NFFT);

%Definerer dempningen til blgeformen
%RTZ-DAC med T=3.68ns
b1=zeros(1,round((1/(40e6))*fs));
b1(1:round(3.68e-9*fs))=1;
%RTZ-DAC med T=3.68ns
b3=zeros(1,round((1/(40e6))*fs));
b3(1:round(4e-9*fs))=1;
%RTZ-DAC med T=5.6ns
b2=zeros(1,round((1/(40e6))*fs));
b2(1:round(5.6e-9*fs))=1;

%Beregner frekvensresponsen til blgeformene
B1=fft(b1,NFFT)/length(b1);
B2=fft(b2,NFFT)/length(b2);
B3=fft(b3,NFFT)/length(b3);

%Plotter frekvensresponsen til RTZ-krets utgangen sammen med
%blgeformdempning, koaksialkabel 1
figure();
plot(f,20*log10(fftshift(abs(X1)./max(abs(X1)))));
hold on;
plot(f,20*log10(fftshift(abs(B1)./max(abs(B1)))),'m');
plot(f,20*log10(fftshift(abs(B3)./max(abs(B3)))),'c--');
title('Frekvensspekter utgang RTZ-krets, NFFT=262144', 'FontSize', FontSize,'FontName',FontName);
xlabel('f [MHz]','FontSize', FontSize,'FontName',FontName);
ylabel('|X(f)| [dB]','FontSize', FontSize,'FontName',FontName);
axis([0 500 -70 2]);
set(gca, 'FontSize', FontSize,'FontName',FontName);
hleg=legend('RTZ-krets med koaksialkabel 1','Dempning RTZ-DAC med T=3.58ns',2);
set(hleg,'FontSize', 12,'FontName','Times New Roman');
hold off;

%Plotter frekvensresponsen til RTZ-krets utgangen sammen med
%blgeformdempning, koaksialkabel 1
figure();
plot(f,20*log10(fftshift(abs(X2)./max(abs(X2)))));
hold on;
plot(f,20*log10(fftshift(abs(B2)./max(abs(B2)))),'m');
title('Frekvensspekter utgang RTZ-krets, NFFT=262144', 'FontSize', FontSize,'FontName',FontName);
xlabel('f [MHz]','FontSize', FontSize,'FontName',FontName);
ylabel('|X(f)| [dB]','FontSize', FontSize,'FontName',FontName);
axis([0 500 -70 2]);
set(gca, 'FontSize', FontSize,'FontName',FontName);
hleg=legend('RTZ-krets med koaksialkabel 2','Dempning RTZ-DAC med T=5.6ns',2);
set(hleg,'FontSize', 12,'FontName','Times New Roman');
hold off;

%% nsker  sammenligne SNR p inngang og utgang!

%Inngang 8MHz
x1=textread('innsignal8MHz_500MS_10us_2pkpk.dat')';
%Fjerner eventuelle DC komponenter
x1=x1-mean(x1);
%Utgang etter BP-filter L6205, med koaksialkabel 1
x2=textread('utsignal_medium_coax_500MS_10us_30mVpkpk.dat')';
%Fjerner eventuelle DC komponenter
x2=x2-mean(x2);
%Utgang etter BP-filter L6205, med koaksialkabel 2
x3=textread('utsignal_lang_coax_500MS_10us_30mVpkpk.dat')';
%Fjerner eventuelle DC komponenter
x3=x3-mean(x3);

%Beregner frekvensresponsen med et Blackman-Harris windu
X1=fft(x1.*window(@blackmanharris,length(x1))');
X2=fft(x2.*window(@blackmanharris,length(x2))');
X3=fft(x3.*window(@blackmanharris,length(x3))');

%Plotter frekvensresponsen for  se hvor signalkomponentene ligger
%figure();
%stem(abs(X2));

%Definerer sty og signal effekter
sign_pow1=0;
noise_pow1=0;

sign_pow2=0;
noise_pow2=0;

sign_pow3=0;
noise_pow3=0;

%Gr igjennom innsignalet og summerer signal og stykomponenter
for i=1:length(X1)
    if i>=77 && i<=85 || i>=4917 && i<=4925  
        sign_pow1=sign_pow1+(abs(X1(i))^2);
    else
        noise_pow1=noise_pow1+abs(X1(i))^2;
   end
end
%Gr igjennom de to utgangssignalene og summerer signal og stykomponenter
for i=1:length(X1)
    if i>=1277 && i<=1285 || i>=3717 && i<=3725  
        sign_pow2=sign_pow2+(abs(X2(i))^2);
        sign_pow3=sign_pow3+(abs(X3(i))^2);
    else
        noise_pow2=noise_pow2+abs(X2(i))^2;
        noise_pow3=noise_pow3+abs(X3(i))^2;
   end
end


%Beregner SNR i alle de tre tilfellene
SNR1=10*log10(sign_pow1/noise_pow1);
SNR2=10*log10(sign_pow2/noise_pow2);
SNR3=10*log10(sign_pow3/noise_pow3);


%Plotter BP filtrert utgang med koaksialkabel 1, sammen med terminert
%sty fra oscilloskopet.

%Terminert inngang p oscilloskopet
x4=textread('terminert_scoop_500MS_10us_1.6mVpkpk.dat')';
%Fjerner eventuelle DC komponenter
x4=x4-mean(x4);

%Punktprvingsfrekvens
fs=500e6;

%nsket lengde til FFT-algoritmen
NFFT=2^(nextpow2(length(x1)));

% Beregner Frekvensspekteret med et Hanning vindu
X1=fft(x1.*window(@hann,length(x2))',NFFT);
X2=fft(x2.*window(@hann,length(x2))',NFFT);
X3=fft(x3.*window(@hann,length(x3))',NFFT);
X4=fft(x4.*window(@hann,length(x4))',NFFT);

%Frekvensvektor til frekvensreponsen
f=linspace(-0.5*(fs/1e6),0.5*(fs/1e6),NFFT);

%plotter innsignalet
figure();
plot(f,20*log10(fftshift(abs(X1)./max(abs(X1)))));
title('Frekvensspekter inngangssignl, NFFT=8192', 'FontSize', FontSize,'FontName',FontName);
xlabel('f [MHz]','FontSize', FontSize,'FontName',FontName);
ylabel('|X(f)| [dB]','FontSize', FontSize,'FontName',FontName);
axis([0 250 -70 2]);
set(gca, 'FontSize', FontSize,'FontName',FontName);

%plotter utsignalet med sty
figure();
plot(f,20*log10(fftshift(abs(X2)./max(abs(X3)))));
hold on;
plot(f,20*log10(fftshift(abs(X4)./max(abs(X3)))),'m');
title('Frekvensspekter bndpassfiltrert utgang, NFFT=8192', 'FontSize', FontSize,'FontName',FontName);
xlabel('f [MHz]','FontSize', FontSize,'FontName',FontName);
ylabel('|X(f)| [dB]','FontSize', FontSize,'FontName',FontName);
axis([0 250 -70 2]);
set(gca, 'FontSize', FontSize,'FontName',FontName);
hleg=legend('Utgang med koaksialkabel 1','Styniv oscilloskop',2);
set(hleg,'FontSize', 12,'FontName','Times New Roman');
hold off;
figure();
plot(f,20*log10(fftshift(abs(X3)./max(abs(X3)))));
hold on;
plot(f,20*log10(fftshift(abs(X4)./max(abs(X3)))),'m');
title('Frekvensspekter bndpassfiltrert utgang, NFFT=8192', 'FontSize', FontSize,'FontName',FontName);
xlabel('f [MHz]','FontSize', FontSize,'FontName',FontName);
ylabel('|X(f)| [dB]','FontSize', FontSize,'FontName',FontName);
axis([0 250 -70 2]);
set(gca, 'FontSize', FontSize,'FontName',FontName);
hleg=legend('Utgang med koaksialkabel 2','Styniv oscilloskop',2);
set(hleg,'FontSize', 12,'FontName','Times New Roman');
hold off;







