%% 2-TONE TESTBENK FOR SAMPLING DOWN CONVERSION 
% DETTE MATLAB SCRIPTET SIMULERER SAMPLING DOWN CONVERSION. ALLER 
% FRST BLIR INNGANGSPARAMETERNE SATT. DERETTER BLI RIKTIG 
% SAMPLINGSFREKVENS SATT UTIFRA INNGANGPARAMETERNE. SOM SAMPLER 
% BLIR EN REELL KRETS SIMULERT, OG PARAMETENE TIL DENNE KRETSEN 
% M SETTES. JITTER VIL OGS INNFRES, DERFOR M RMS VERDIEN 
% AV DENNE SETTES. FRST SIMULERES EN IDEELL UNDERSAMPLIG, 
% DERETTER INNFRES EN MER REELL SAMPLER. S BLIR JITTERS 
% PVIRKNING P PASSBNDSSIGNALET SIMULERT, FR REELL SAMPLER 
% OG JITTER SIMULERES SAMMEN HELT TIL SLUTT.

clear all;
close all;

% INNGANGSPARAMETERE
F1=0.7e+9;          %Testsignalfrekvenser
F2=0.6e+9;          %Testsignalfrekvenser
Fc=9e+9;            %Breblgefrekvens
B=1e9;              %Omhylningsbndbredde bndbredde
FL=Fc;              %Laveste frekvens i omhylningen
FH=FL+B;            %Hyeste frekvens i omhylningen
tidsluke=1e-6;      %Varighet p testssignalet

% VALG AV SAMPLINGSFREKVENS 

%Finner hyeste NQ-bnd k, og evt. korrigerer til oddetall
k=floor(FH/B);
if mod(k,2)
    k=k;
else
    k=k-1;
end

%Setter riktig samplingsfrekvens.
fs=2*FL/(k-1);      %Reell samplingsfrekvens for undersampling
fsa=10*fs;          %Fiktiv "analog" samplingsfrekvens

% PARAMETERE TIL REELL SAMPLER
fco=20e+9;              %cut-off frekvens for RC-filter-sampleren.
tau=1/(2*pi*fco);       %tidskonstant
AT=4*tau;               %Acquisition time

%nsket opplsning i sampleren
n_ps=10;                %#punktprver/ps
n_s=n_ps*1e+12;         %#punktprver/s

% JITTER
jitterRMS=1e-12;      %RMS verdien av jitter
jitter=(jitterRMS).*randn(1,length((1/fs):1/fs:tidsluke-(1/fs)));

%Font til plott
FontSize=16;
FontName='Times New Roman';

%% IDEELL UNDERSAMPLING

%Genererer en "analog" tidsvektor
t=0:(1/fsa):(tidsluke)-(1/fsa);

%Basisbndssignalet
xb=sin((2*pi)*F1.*t)+sin((2*pi)*F2.*t);

%Flytter signalet opp i frekvens til passbnd
xrf=sin((2*pi)*(F1+Fc).*t)+sin((2*pi)*(F2+Fc).*t);

%Undersampler passbndssignalet signalet + setter 0 mellom alle punktrpvene
x=zeros(1,length(xrf));
for i=1:fsa/fs:length(xrf)
    x(i)=xrf(i);
end

%Beregner Fouriertransformen
Xb=fft(xb.*window(@hann,length(xb))',2^(nextpow2(length(xb))+2));
Xrf=fft(xrf.*window(@hann,length(xrf))',2^(nextpow2(length(xrf))+2));
X=fft(x.*window(@hann,length(x))',2^(nextpow2(length(x))+2));

%Plotter basisbndssignalet
figure();
plot(linspace(-0.5*fsa/1e9,0.5*fsa/1e9,length(Xb)),20*log10(fftshift(abs(Xb))./max(abs(Xb))),'g');
title('Basebndsignal', 'FontSize', FontSize,'FontName',FontName);
xlabel('F[GHz]','FontSize', FontSize,'FontName',FontName);
ylabel('|Xb| [dB]','FontSize', FontSize,'FontName',FontName);
axis([0 fs/2e9 -40 2]);
set(gca, 'FontSize', FontSize,'FontName',FontName);

%Ploter RF-signalet
figure();
plot(linspace(-0.5*fsa/1e9,0.5*fsa/1e9,length(Xrf)),20*log10(fftshift(abs(Xrf))./max(abs(Xrf))),'r');
title('RF-signal', 'FontSize', FontSize,'FontName',FontName);
xlabel('F[GHz]','FontSize', FontSize,'FontName',FontName);
ylabel('|Xrf| [dB]','FontSize', FontSize,'FontName',FontName);
axis([8*fs/2e9 9*fs/2e9 -40 2]);
set(gca, 'FontSize', FontSize,'FontName',FontName);

%Plotter undersamplet signal med 0 mellom alle punktprver
figure();
plot(linspace(-0.5*fsa/1e9,0.5*fsa/1e9,length(X)),20*log10(fftshift(abs(X))./max(abs(X))),'b');
title(['Undersamplet signal, fs= ' num2str(fs/1e+9) 'GHz'], 'FontSize', FontSize,'FontName',FontName);
xlabel('F[GHz]','FontSize', FontSize,'FontName',FontName);
ylabel('|X| [dB]','FontSize', FontSize,'FontName',FontName);
axis([0 10*fs/2e9 -40 2]);
set(gca, 'FontSize', FontSize,'FontName',FontName);

%% UNDERSAMPLER MED REELL SAMPLER
%(For ikke  generere for mye data blir kun dataene rundt de nskede
% punktprvene generert. Selve samplingen skjer i MATLAB scriptet sample.m)

%genererer nsket tidsvektor (henter kun ut verdiene rundt fs)
t=zeros(1,round(n_s*AT)); %NB! 1. sampel blir ideelt    

for i=(1/fs):1/fs:tidsluke-(1/fs)   
    t=[t (i-(round(n_s*AT)-1)/n_s):1/n_s:i];
end

%genererer det "analoge" signalet rundt punktprvene
xrf=sin((2*pi)*(F1+Fc).*t)+sin((2*pi)*(F2+Fc).*t);

%undersampler med "reell" sampler
x_sample=zeros(1,tidsluke/(1/fs));
teller=0;
for i=1:round(n_s*AT):length(t)
    teller=teller+1;
    x_sample(teller)=sample(xrf(i:i+round(n_s*AT)-1),AT,fco);
end

%Plotter et tidsutsnitt
t_utsnitt=1e-9;
figure();
plot(0:1/fsa:t_utsnitt,xb(1:length(0:1/fsa:t_utsnitt)),'g');
hold on;
stem(0:1/fs:t_utsnitt,x_sample(1:length(0:1/fs:t_utsnitt)));
for i=1:length(0:1/fs:t_utsnitt)-1
    plot(t(i*round(n_s*AT)+1:(i+1)*round(n_s*AT)),xrf(i*round(n_s*AT)+1:(i+1)*round(n_s*AT)),'r');
end
hold off;
title(['"Reell" punktprvning med AT = ' num2str(AT/(tau)) '\tau, F_{co}= ' num2str(fco/1e9) 'GHz, F_{s}= ' num2str(fs/1e9) 'GHz'],'FontSize', 16,'FontName','Calibri');
xlabel('t[ns]','FontSize', 16,'FontName','Times New Roman');
ylabel('x(t)','FontSize', 16,'FontName','Times New Roman');
hleg=legend('Basebndsignal','Sampel','Passbndsignal gjennom RC-filter',3);
set(hleg,'FontSize', 12,'FontName','Times New Roman');
set(gca, 'FontSize', 16,'FontName','Times New Roman');

%plotter frekvensresponsen til det undersamplede signalet, 
%nomaliser til en ideell undnersampling

%Beregner Fouriertransformen
X_SAMPLE=fft(x_sample.*window(@hann,length(x_sample))',2^(nextpow2(length(x_sample))+2));

%Plotter undersamplet signal med 0 mellom alle punktprver
figure();
plot(linspace(-0.5*fs/1e9,0.5*fs/1e9,length(X_SAMPLE)),20*log10(fftshift(abs(X_SAMPLE))./max(abs(X))),'b');
title(['Frekvensspekter, AT = ' num2str(AT/tau) '\tau og Fco= ' num2str(fco/1e9) 'GHz'], 'FontSize', FontSize,'FontName',FontName);
xlabel('F[GHz]','FontSize', FontSize,'FontName',FontName);
ylabel('|X| [dB]','FontSize', FontSize,'FontName',FontName);
axis([0 1*fs/2e9 -10 2]);
set(gca, 'FontSize', FontSize,'FontName',FontName);

%% SER P VIRKNINGEN AV JITTER P PASSBNDSIGNALET
%(Denne simuleringen er uavhengig av de foregende simuleringene, og
%har kun som hensikt  se p jitters innvirkning p et passbndssignal)

%Teoretisk jitter
t_jitter=1e-12.*[0.1:0.1:10];
SNR_TEO=-20*log10(2*pi*(F1+Fc).*t_jitter);

%Ideelt undersamplet testssignal
t=0:1/fs:tidsluke-(1/fs);
x=sin((2*pi)*(F1+Fc).*t)+sin((2*pi)*(F2+Fc).*t);

%NSKER  LEGGE P JITTER

%Hjelpevektorer
tj=zeros(1,length(t));
xj=zeros(1,length(x));
SNR=zeros(1,length(t_jitter));

%regner ut SNR for hver av RMS jitter verdiene
for j=1:length(t_jitter)
    %Gr igjennom tidsvektoren og legger p jitter
    for i=1:length(t)    
        tj(i)=t(i)+(t_jitter(j)*randn(1));    
    end
    %Genererer signalet med jitter
    xj=sin((2*pi)*(F1+Fc).*tj)+sin((2*pi)*(F2+Fc).*tj);
    %finner feilen generert av jitter
    e=x-xj;
    %regner ut SNR
    SNR(j)=10*log10(mean(x.^2)/mean(e.^2));
end

%plotter simulert jitter kontra teoretisk jitter
figure();
plot(t_jitter./(1e-12),SNR);
hold on;
plot(t_jitter./(1e-12),SNR_TEO,'r');
hold off;
grid on;
title(['SNR vs. RMS jitter, maks \Delta' 'SNR=' num2str(max(abs(SNR_TEO-SNR)),2) ' dB'], 'FontSize', 16,'FontName','Times New Roman');
xlabel('t_{j} [ps]', 'FontSize', 16,'FontName','Times New Roman');
set(gca,'xscale','log');
ylabel('SNR [dB]', 'FontSize', 16,'FontName','Times New Roman');
hleg=legend('utregnet SNR','teoretisk SNR',2);
set(hleg,'FontSize', 12,'FontName','Times New Roman');
set(gca, 'FontSize', 16,'FontName','Times New Roman');

%plotter frekvensresponsen til undersamplet signal med 
%3 forskjellige RMS jitter verdier
jitterRMS=1e-12.*[10 1 0.1];

for j=1:length(jitterRMS)
    %Gr igjennom tidsvektoren og legger p jitter
    for i=1:length(t)    
        tj(i)=t(i)+(jitterRMS(j)*randn(1));    
    end
    %Genererer signalet med jitter
    xj=sin((2*pi)*(F1+Fc).*tj)+sin((2*pi)*(F2+Fc).*tj);
    %Beregner Fouriertransformen
    Xj=fft(xj.*window(@hann,length(xj))',2^(nextpow2(length(xj))+2));

    %Plotter undersamplet signal med 0 mellom alle punktprver
    figure();
    plot(linspace(-0.5*fs/1e9,0.5*fs/1e9,length(Xj)),20*log10(fftshift(abs(Xj))./max(abs(Xj))),'b');
    title(['Frekvensspekter, t_{j}= ' num2str(jitterRMS(j)/1e-12) 'ps, NFFT=2^{' num2str((nextpow2(length(xj))+2)) '}'], 'FontSize', FontSize,'FontName',FontName);
    xlabel('F[GHz]','FontSize', FontSize,'FontName',FontName);
    ylabel('|Xj| [dB]','FontSize', FontSize,'FontName',FontName);
    axis([0 1*fs/2e9 -80 2]);
    set(gca, 'FontSize', FontSize,'FontName',FontName); 
end

%% SER P VIRKNINGEN AV JITTER OG REELL SAMPLER SAMMEN

%undersampler med "reell" sampler
x_sample=zeros(1,tidsluke/(1/fs));
teller=0;
for i=1:round(n_s*AT):length(t)
    teller=teller+1;
    x_sample(teller)=sample(xrf(i:i+round(n_s*AT)-1),AT,fco);
end

%genererer tidsvektor med og uten jitter (henter kun ut verdiene rundt fs)
t=zeros(1,round(n_s*AT)); 
tj=zeros(1,round(n_s*AT));
teller=1;
for i=(1/fs):1/fs:tidsluke-(1/fs)
    t=[t (i-(round(n_s*AT)-1)/n_s):(1/n_s):i];
    tj=[tj (i-(round(n_s*AT)-1)/n_s)+jitter(teller):(1/n_s):i+jitter(teller)];
    teller=teller+1;
end

%genererer det "analoge" signalet uten og med jitter
x=sin((2*pi)*(F1+Fc).*t)+sin((2*pi)*(F2+Fc).*t);
xj=sin((2*pi)*(F1+Fc).*tj)+sin((2*pi)*(F2+Fc).*tj);

%undersampler signalet med og uten jitter
x_sample=zeros(1,tidsluke/(1/fs)); 
x_samplej=zeros(1,tidsluke/(1/fs));
teller=0;
for i=1:length(t)/(tidsluke/(1/fs)):length(t)
    teller=teller+1;
    x_sample(teller)=sample(x(i:i+round(n_s*AT)-1),AT,fco);               
    x_samplej(teller)=sample(xj(i:i+round(n_s*AT)-1),AT,fco);                    
end

%Beregner SNR fr og etter sampling 
e1=x-xj;                  %feil fr sampling
e2=x_sample-x_samplej;      %feil etter sampling
SNRfor=10*log10(mean(x.^2)/mean(e1.^2));
SNRetter=10*log10(mean(x_sample.^2)/mean(e2.^2));

%Beregner Fouriertransformen
X_SAMPLEJ=fft(x_samplej.*window(@hann,length(x_samplej))',2^(nextpow2(length(x_samplej))+2));

%Plotter frekvensspekteret, normalisert til den ideelle undersamplingen.
figure();
plot(linspace(-0.5*fs/1e9,0.5*fs/1e9,length(X_SAMPLEJ)),20*log10(fftshift(abs(X_SAMPLEJ))./max(abs(X))),'b');
title(['Frekvensspekter, AT = ' num2str(AT/tau) '\tau og Fco= ' num2str(fco/1e9) 'GHz'], 'FontSize', FontSize,'FontName',FontName);
xlabel('F[GHz]','FontSize', FontSize,'FontName',FontName);
ylabel('|Xj| [dB]','FontSize', FontSize,'FontName',FontName);
axis([0 1*fs/2e9 -60 2]);
set(gca, 'FontSize', FontSize,'FontName',FontName);