%% 2-TONE TESTBENK FOR SAMPLING UP CONVERSION 
% DETTE MATLAB SCRIPTET SIMULERER SAMPLING UP CONVERSION. ALLER FRST 
% BLIR INNGANGSPARAMETERNE SATT, DERETTER RIKTIG SAMPLINGSFREKVENS
% UTIFRA INNGANGPARAMETERNE. PARAMETERNE FOR RTZ-DAC BLIR OGS SATT. 
% FRST BLIR BLGEFORMENE STUDERT, DERETTER LAGT TIL BASEBNDSIGN.
% DERETTER KOMMER EN REELL TILNRMING TIL BLGEFORMEN, FR 
% JITTER INNFRES I NEDKONVERTERINGEN. TIL SLUTT BLIR ALTERNATIVE
% BLGEFORMER STUDERT

clear all;
close all;

% INNGANGSPARAMETERE
F1=0.7e+9;          %Testsignalfrekvenser
F2=0.6e+9;          %Testsignalfrekvenser
Fc=9e+9;            %Breblgefrekvens
B=1e9;              %Omhylningsbndbredde bndbredde
FL=Fc;              %Laveste frekvens i omhylningen
FH=FL+B;            %Hyeste frekvens i omhylningen
tidsluke=1e-6;      %Varighet p testssignalet


% VALG AV SAMPLINGSFREKVENS 
%Finner hyeste NQ-bnd k, og evt. korrigerer til oddetall
k=floor(FH/B);
if mod(k,2)
    k=k;
else
    k=k-1;
end

fs=2*FL/(k-1);       %Reell samplingsfrekvens for undersampling
fsa=400*fs;          %Fiktiv "analog" samplingsfrekvens

% PARAMETERE FOR RTZ-DAC
T=[50 100 150 250]*1e-12;              %(NB! T<=Ts=1/Fs)
RMSjitter=10e-12;

%Font til plott
FontSize=16;
FontName='Times New Roman';

%% IDEELL RTZ-DAC OPPKONVERTERING
%Genererer en "analog" tidsvektor
t=0:(1/fs):(tidsluke)-(1/fs);
%Basisbndssignalet
xb=sin((2*pi)*F1.*t)+sin((2*pi)*F2.*t);

%Blgeformer
%ZERO-ORDER-HOLD
zoh=ones(1,round((1/fs)*fsa));
%RETURN-TO-ZERO
rtz_50=zeros(1,round((1/fs)*fsa));
rtz_50(1:round(T(1)*fsa))=1;
rtz_100=zeros(1,round((1/fs)*fsa));
rtz_100(1:round(T(2)*fsa))=1;
rtz_150=zeros(1,round((1/fs)*fsa));
rtz_150(1:round(T(3)*fsa))=1;
rtz_250=zeros(1,round((1/fs)*fsa));
rtz_250(1:round(T(4)*fsa))=1;

%Tidsutsnitt
t_bolge=[0:1/fsa:1/fs-1/fsa].*(fs);

%Regner ut frekvensresponsen til blgeformene
NFFT=10*2048;
f=linspace(-0.5*(fsa/1e9),0.5*(fsa/1e9),NFFT);
ZOH=fft(zoh,NFFT)/length(zoh);
RTZ_50=fft(rtz_50,NFFT)/length(rtz_50);
RTZ_100=fft(rtz_100,NFFT)/length(rtz_100);
RTZ_150=fft(rtz_150,NFFT)/length(rtz_150);
RTZ_250=fft(rtz_250,NFFT)/length(rtz_250);

%Plotter blgeformene sammen med tilhrende frekvensspekter.
figure();
subplot(4,2,1);
plot(t_bolge,zoh);
title('Blgeformer', 'FontSize', FontSize,'FontName',FontName);
axis([0 1 0 1.2]);
set(gca, 'FontSize', FontSize,'FontName',FontName);
ylabel('T=T_{s}','FontSize', FontSize,'FontName',FontName);
subplot(4,2,2);
plot(f,20*log10(fftshift(abs(ZOH))));
title('Frekvensspekter [dB]', 'FontSize', FontSize,'FontName',FontName);
axis([0 25 -50 3]);
set(gca, 'FontSize', FontSize,'FontName',FontName);

subplot(4,2,3);
plot(t_bolge,rtz_250);
axis([0 1 0 1.2]);
set(gca, 'FontSize', FontSize,'FontName',FontName);
ylabel(['T=' num2str(T(4)/1e-12) 'ps'],'FontSize', FontSize,'FontName',FontName);
subplot(4,2,4);
plot(f,20*log10(fftshift(abs(RTZ_250))));
axis([0 25 -50 3]);
set(gca, 'FontSize', FontSize,'FontName',FontName);

subplot(4,2,5);
plot(t_bolge,rtz_100);
axis([0 1 0 1.2]);
set(gca, 'FontSize', FontSize,'FontName',FontName);
ylabel(['T=' num2str(T(2)/1e-12) 'ps'],'FontSize', FontSize,'FontName',FontName);
subplot(4,2,6);
plot(f,20*log10(fftshift(abs(RTZ_100))));
axis([0 25 -50 3]);
set(gca, 'FontSize', FontSize,'FontName',FontName);

subplot(4,2,7);
plot(t_bolge,rtz_50);
axis([0 1 0 1.2]);
set(gca, 'FontSize', FontSize,'FontName',FontName);
xlabel('Tid i Ts sekunder','FontSize', FontSize,'FontName',FontName);
ylabel(['T=' num2str(T(1)/1e-12) 'ps'],'FontSize', FontSize,'FontName',FontName);
subplot(4,2,8);
plot(f,20*log10(fftshift(abs(RTZ_50))));
axis([0 25 -50 3]);
set(gca, 'FontSize', FontSize,'FontName',FontName);
xlabel('Frekvens [GHz]','FontSize', FontSize,'FontName',FontName);

%Plotter ZOH og RTZ frekvensresponsene i samme plott
figure();
hold on;
plot(f,20*log10(fftshift(abs(ZOH))),'b');
plot(f,20*log10(fftshift(abs(RTZ_50))),'r');
axis([0 25 -50 3]);
set(gca, 'FontSize', FontSize,'FontName',FontName);
xlabel('Frekvens [GHz]','FontSize', FontSize,'FontName',FontName);
ylabel('Dempning [dB]','FontSize', FontSize,'FontName',FontName);
title('Frekvensdempning av blgeformer', 'FontSize', FontSize,'FontName',FontName);
hleg=legend('ZOH-DAC','RTZ-DAC, T=50ps',2);
set(hleg,'FontSize', 12,'FontName','Times New Roman');
hold off;

%Plotter alle frekvensresponsene i samme plott
figure();
hold on;
plot(f,20*log10(fftshift(abs(RTZ_50))),'b');
plot(f,20*log10(fftshift(abs(RTZ_100))),'r');
plot(f,20*log10(fftshift(abs(RTZ_150))),'g');
plot(f,20*log10(fftshift(abs(RTZ_250))),'k');
axis([0 10 -50 3]);
set(gca, 'FontSize', FontSize,'FontName',FontName);
xlabel('Frekvens [GHz]','FontSize', FontSize,'FontName',FontName);
ylabel('Dempning [dB]','FontSize', FontSize,'FontName',FontName);
title('Frekvensdempning av blgeformer', 'FontSize', FontSize,'FontName',FontName);
hleg=legend('T=50ps','T=100ps','T=150ps','T=250ps',4);
set(hleg,'FontSize', 12,'FontName','Times New Roman');
hold off;

%Legger til blgeformen p alle punktprvene i basebndsignalet.
xzoh=DAComformer(xb,zoh);
XZOH=fft(xzoh.*window(@hann,length(xzoh))',2^(nextpow2(length(xzoh))+2));
figure();
plot(linspace(-0.5*(fsa/1e9),0.5*(fsa/1e9),length(XZOH)),20*log10(fftshift(abs(XZOH)/max(abs(XZOH))))+20*log10(sinc(600e6*(1/fs))));
hold on;
plot(linspace(-0.5*(fsa/1e9),0.5*(fsa/1e9),length(ZOH)),20*log10(fftshift(abs(ZOH))),'m');
hold off;
set(gca, 'FontSize', FontSize,'FontName',FontName);
xlabel('Frekvens [GHz]','FontSize', FontSize,'FontName',FontName);
ylabel('Dempning [dB]','FontSize', FontSize,'FontName',FontName);
title('Frekvensspekter ZOH-DAC, fs=2.25GHz', 'FontSize', FontSize,'FontName',FontName);
hleg=legend('DAC-utgang','Blgeformdempning',2);
set(hleg,'FontSize', 12,'FontName','Times New Roman');
axis([0 25 -50 2]);

xrtz=DAComformer(xb,rtz_50);
XRTZ=fft(xrtz.*window(@hann,length(xrtz))',2^(nextpow2(length(xrtz))+2));
figure();
plot(linspace(-0.5*(fsa/1e9),0.5*(fsa/1e9),length(XRTZ)),20*log10(fftshift(abs(XRTZ)/max(abs(XZOH))))+20*log10(sinc(600e6*(1/fs))));
hold on;
plot(linspace(-0.5*(fsa/1e9),0.5*(fsa/1e9),length(RTZ_50)),20*log10(fftshift(abs(RTZ_50))),'m');
hold off;
set(gca, 'FontSize', FontSize,'FontName',FontName);
xlabel('Frekvens [GHz]','FontSize', FontSize,'FontName',FontName);
ylabel('Dempning [dB]','FontSize', FontSize,'FontName',FontName);
title('Frekvensspekter RTZ-DAC, T=50ps, fs=2.25GHz', 'FontSize', FontSize,'FontName',FontName);
hleg=legend('DAC-utgang','Blgeformdempning',2);
set(hleg,'FontSize', 12,'FontName','Times New Roman');
axis([0 25 -50 2]);
%% REELL TILNRMING P BLGEFORM

%RETURN-TO-ZERO-MED-REELL-BLGEFORM
rtz_r1=zeros(1,round((1/fs)*fsa));
rtz_r2=zeros(1,round((1/fs)*fsa));
rtz_r3=zeros(1,round((1/fs)*fsa));
rtz_r4=zeros(1,round((1/fs)*fsa));

rtz_r1(1:round(T(1)*fsa))=1;
rtz_r2(1:round(T(1)*fsa))=1;
rtz_r3(1:round(T(1)*fsa))=1;
rtz_r4(1:round(T(1)*fsa))=1;
    
%Forlp 
t_tmp=0:1/fsa:50e-12;

%RC- og RCL-forlp
tau_tmp=5e-12;
forlop1=1-exp(-t_tmp./tau_tmp);
forlop2=exp(-t_tmp./tau_tmp);
demp=0.4;
w0=400e9;
wd=w0*sqrt(1-demp^2);
alpha=w0*demp;
forlop3=1+exp(-alpha.*t_tmp).*(-cos(wd.*t_tmp)+0.1*sin(wd.*t_tmp));

%Alternative blgeformer
rtz_r1(1:length(forlop1))=forlop1;
rtz_r2(round(T(1)*fsa):round(T(1)*fsa)+length(t_tmp)-1)=forlop2;
rtz_r3(1:length(forlop3))=forlop3;
rtz_r4(1:length(forlop1))=forlop3;
rtz_r4(round(T(1)*fsa):round(T(1)*fsa)+length(t_tmp)-1)=forlop2;
%Frekvensresponsen til de reelle blgeformene
RTZ_R1=fft(rtz_r1,NFFT)/length(rtz_r1);
RTZ_R2=fft(rtz_r2,NFFT)/length(rtz_r2);
RTZ_R3=fft(rtz_r3,NFFT)/length(rtz_r3);
RTZ_R4=fft(rtz_r4,NFFT)/length(rtz_r4);

%Frekvens og tidsvektor
f=linspace(-0.5*(fsa/1e9),0.5*(fsa/1e9),NFFT);
t_bolge=[0:1/fsa:1/fs-1/fsa].*1e12;

%Plotter alle tidsresponsene
figure();
FontSize=16;
FontName='Times New Roman';
subplot(2,2,1);
plot(t_bolge,rtz_r1,'b');
title('Blgeform 1', 'FontSize', FontSize,'FontName',FontName);
axis([0 80 0 1.3]);
set(gca, 'FontSize', FontSize,'FontName',FontName);
xlabel('Tid [ps]','FontSize', 12,'FontName',FontName);

subplot(2,2,2);
plot(t_bolge,rtz_r2,'r');
title('Blgeform 2', 'FontSize', FontSize,'FontName',FontName);
axis([0 80 0 1.3]);
set(gca, 'FontSize', FontSize,'FontName',FontName);
xlabel('Tid [ps]','FontSize', 12,'FontName',FontName);

subplot(2,2,3);
plot(t_bolge,rtz_r3,'g');
title('Blgeform 3', 'FontSize', FontSize,'FontName',FontName);
axis([0 80 0 1.3]);
set(gca, 'FontSize', FontSize,'FontName',FontName);
xlabel('Tid [ps]','FontSize', 12,'FontName',FontName);

subplot(2,2,4);
plot(t_bolge,rtz_r4,'k');
title('Blgeform 4', 'FontSize', FontSize,'FontName',FontName);
axis([0 80 0 1.3]);
set(gca, 'FontSize', FontSize,'FontName',FontName);
xlabel('Tid [ps]','FontSize', 12,'FontName',FontName);

%PLotter alle frekvensresponsene
figure();
hold on;
plot(f,20*log10(fftshift(abs(RTZ_R1))),'b');
plot(f,20*log10(fftshift(abs(RTZ_R2))),'r');
plot(f,20*log10(fftshift(abs(RTZ_R3))),'g');
plot(f,20*log10(fftshift(abs(RTZ_R4))),'k');
axis([0 25 -50 -15]);
hleg=legend('Blgeform 1','Blgeform 2','Blgeform 3','Blgeform 4',4);
set(hleg,'FontSize', 12,'FontName','Times New Roman','Location','NorthEast');
xlabel('Frekvens [GHz]','FontSize', FontSize,'FontName',FontName);
ylabel('Dempning','FontSize', FontSize,'FontName',FontName);
set(gca, 'FontSize', FontSize,'FontName',FontName);
title('Frekvensdempning for Blgeformer','FontSize', FontSize,'FontName',FontName);

%% SUC MED JITTER

%Genererer en ny jittret tidsvektor
tj=t+RMSjitter*randn(1,length(t)); 

%Generere jittret basisbndssignal
xbj=sin((2*pi)*F1.*tj)+sin((2*pi)*F2.*tj);

%DAC OMDFORMER
%Legger til blgeformen p alle punktprvene i basebndsignalet.
xrtzj=DAComformer(xbj,rtz_50);
xzohj=DAComformer(xbj,zoh);

%Beregner Fouriertransformen
Xrtzj=fft(xrtzj.*window(@hann,length(xrtzj))',2^(nextpow2(length(xrtzj))+2));

%Plotter Fouriertransformen
figure();
plot(linspace(-0.5*fsa/1e9,0.5*fsa/1e9,length(Xrtzj)),20*log10(fftshift(abs(Xrtzj))/max(abs(Xrtzj))));
set(gca, 'FontSize', FontSize,'FontName',FontName);
xlabel('Frekvens [GHz]','FontSize', FontSize,'FontName',FontName);
ylabel('|X_{rtz-j}|','FontSize', FontSize,'FontName',FontName);
title(['Frekvensspekter for RTZ-DAC, T= ' num2str(T(1)./1e-12) 'ps, fs= ' num2str(fs/1e+9) 'GHz'], 'FontSize', FontSize,'FontName',FontName);
axis([0 25 -100 2]);

%Beregner SNR
e=xb-xbj;                          
SNR=10*log10(mean(xb.^2)/mean(e.^2));

%% Finner ut hva er SNR i basebnd og RF bnd

%Bndpassfiltrerer RTZ-DAC utgangen med 2 forskjellige filtre 
xRF=filter(BP_9_10_order8,xrtzj);
xRF_2=filter(BP_9_10_order40,xrtzj);

%Beregner Frekvensspekter med Blackmann-Harris vindu
XRF=fft(xRF.*window(@blackmanharris,length(xRF))');
XRF_2=fft(xRF_2.*window(@blackmanharris,length(xRF_2))');

%Plotter for  finne riktige frekvenskomponenter
figure();
stem(abs(XRF));

%Definerer signal og sty vektorer
sRF=0;nRF=0;
sRF_2=0;nRF_2=0;

%Gr igjennom og kvaderer og summerer signal og sty bidrag
%NB! Gr kun igjennom halve, da frekvensspekteret er speilet.
for i=1:length(XRF)/2
    if i>=9597 && i<=9605 || i>=9697 && i<=9705  
        sRF=sRF+abs(XRF(i))^2;
        sRF_2=sRF_2+abs(XRF_2(i))^2;
    else
        nRF=nRF+abs(XRF(i))^2;
        nRF_2=nRF_2+abs(XRF_2(i))^2;
   end
end

%Beregner SNR
SNR_filterRF=10*log10(sRF/nRF);
SNR_filterRF_2=10*log10(sRF_2/nRF_2);
%% STUDERER ANDRE BLGEFORMER

%ZOH blgeform
zoh=ones(1,round((1/fs)*fsa));

%Forming av alternativ blgeform
rtz_50=zeros(1,round((1/fs)*fsa));
rtz_50(1:round(T(1)*fsa))=1;
rtz_50(round(T(1)*fsa)+1:2*round(T(1)*fsa))=-1;
rtz_50(2*round(T(1)*fsa)+1:3*round(T(1)*fsa))=1;
rtz_50(3*round(T(1)*fsa)+1:4*round(T(1)*fsa))=-1;
rtz_50(4*round(T(1)*fsa)+1:5*round(T(1)*fsa))=1;
rtz_50(5*round(T(1)*fsa)+1:6*round(T(1)*fsa))=-1;
rtz_50(6*round(T(1)*fsa)+1:7*round(T(1)*fsa))=1;
rtz_50(7*round(T(1)*fsa)+1:8*round(T(1)*fsa))=-1;
rtz_50(8*round(T(1)*fsa)+1:end)=1;

%Plotter tidsresponsen
t_bolge=[0:1/fsa:1/fs-1/fsa].*(fs);
figure();
plot(t_bolge,rtz_50);

%Regner ut frekvensresponsen til blgeformene og plotter den
NFFT=10*2048;
f=linspace(-0.5*(fsa/1e9),0.5*(fsa/1e9),NFFT);
ZOH=fft(zoh,NFFT)/length(zoh);
RTZ_50=fft(rtz_50,NFFT)/length(rtz_50);
figure();
plot(f,20*log10(fftshift(abs(RTZ_50))),'b');
axis([0 25 -50 3]);
